@@ -50,7 +50,7 @@ static inline uint16_t get_manufacturer(void)
 	return (manufacturer == DEFAULT_COMPID ? parser.defcompid : manufacturer);
 }
 
-#define EVENT_NUM 55
+#define EVENT_NUM 61
 static char *event_str[EVENT_NUM + 1] = {
 	"Unknown",
 	"Inquiry Complete",
@@ -100,17 +100,23 @@ static char *event_str[EVENT_NUM + 1] = {
 	"Synchronous Connect Changed",
 	"Sniff Subrate",
 	"Extended Inquiry Result",
-	"Unknown",
-	"Unknown",
-	"Unknown",
-	"Unknown",
-	"Unknown",
-	"Unknown",
+	"Encryption Key Refresh Complete",
+	"IO Capability Request",
+	"IO Capability Response",
+	"User Confirmation Request",
+	"User Passkey Request",
+	"Remote OOB Data Request",
+	"Simple Pairing Complete",
 	"Unknown",
 	"Link Supervision Timeout Change",
+	"Enhanced Flush Complete",
+	"Unknown",
+	"User Passkey Notification",
+	"Keypress Notification",
+	"Remote Host Supported Features Notification",
 };
 
-#define CMD_LINKCTL_NUM 41
+#define CMD_LINKCTL_NUM 52
 static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Unknown",
 	"Inquiry",
@@ -144,7 +150,7 @@ static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Read Remote Version Information",
 	"Unknown",
 	"Read Clock Offset",
-	"Read LMP Handle"
+	"Read LMP Handle",
 	"Unknown",
 	"Unknown",
 	"Unknown",
@@ -155,6 +161,16 @@ static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Setup Synchronous Connection",
 	"Accept Synchronous Connection",
 	"Reject Synchronous Connection",
+	"IO Capability Request Reply",
+	"User Confirmation Request Reply",
+	"User Confirmation Request Negative Reply",
+	"User Passkey Request Reply",
+	"User Passkey Request Negative Reply",
+	"Remote OOB Data Request Reply",
+	"Unknown",
+	"Unknown",
+	"Remote OOB Data Request Negative Reply",
+	"IO Capability Request Negative Reply",
 };
 
 #define CMD_LINKPOL_NUM 17
@@ -176,10 +192,10 @@ static char *cmd_linkpol_str[CMD_LINKPOL_NUM + 1] = {
 	"Read Default Link Policy Settings",
 	"Write Default Link Policy Settings",
 	"Flow Specification",
-	"Sniff Subrate",
+	"Sniff Subrating",
 };
 
-#define CMD_HOSTCTL_NUM 82
+#define CMD_HOSTCTL_NUM 95
 static char *cmd_hostctl_str[CMD_HOSTCTL_NUM + 1] = {
 	"Unknown",
 	"Set Event Mask",
@@ -264,6 +280,20 @@ static char *cmd_hostctl_str[CMD_HOSTCTL_NUM + 1] = {
 	"Unknown",
 	"Read Extended Inquiry Response",
 	"Write Extended Inquiry Response",
+	"Refresh Encryption Key",
+	"Unknown",
+	"Read Simple Pairing Mode",
+	"Write Simple Pairing Mode",
+	"Read Local OOB Data",
+	"Read Inquiry Response Transmit Power Level",
+	"Write Inquiry Response Transmit Power Level",
+	"Read Default Erroneous Data Reporting",
+	"Write Default Erroneous Data Reporting",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Enhanced Flush"
+	"Unknown",
 };
 
 #define CMD_INFO_NUM 9
@@ -292,7 +322,16 @@ static char *cmd_status_str[CMD_STATUS_NUM + 1] = {
 	"Read Clock",
 };
 
-#define ERROR_CODE_NUM 53
+#define CMD_TESTING_NUM 4
+static char *cmd_testing_str[CMD_TESTING_NUM + 1] = {
+	"Unknown",
+	"Read Loopback Mode",
+	"Write Loopback Mode",
+	"Enable Device Under Test mode",
+	"Unknown",
+};
+
+#define ERROR_CODE_NUM 56
 static char *error_code_str[ERROR_CODE_NUM + 1] = {
 	"Success",
 	"Unknown HCI Command",
@@ -348,6 +387,9 @@ static char *error_code_str[ERROR_CODE_NUM + 1] = {
 	"Reserved",
 	"Reserved Slot Violation",
 	"Role Switch Failed",
+	"Extended Inquiry Response Too Large",
+	"Simple Pairing Not Supported by Host",
+	"Host Busy - Pairing",
 };
 
 static char *status2str(uint8_t status)
@@ -405,7 +447,10 @@ static char *opcode2str(uint16_t opcode)
 		break;
 
 	case OGF_TESTING_CMD:
-		cmd = "Testing";
+		if (ocf <= CMD_TESTING_NUM)
+			cmd = cmd_testing_str[ocf];
+		else
+			cmd = "Unknown";
 		break;
 
 	case OGF_VENDOR_CMD:
@@ -420,6 +465,20 @@ static char *opcode2str(uint16_t opcode)
 	return cmd;
 }
 
+static char *linktype2str(uint8_t type)
+{
+	switch (type) {
+	case 0x00:
+		return "SCO";
+	case 0x01:
+		return "ACL";
+	case 0x02:
+		return "eSCO";
+	default:
+		return "Unknown";
+	}
+}
+
 static char *role2str(uint8_t role)
 {
 	switch (role) {
@@ -464,6 +523,64 @@ static char *airmode2str(uint8_t mode)
 	}
 }
 
+static char *keytype2str(uint8_t type)
+{
+	switch (type) {
+	case 0x00:
+		return "Combination Key";
+	case 0x01:
+		return "Local Unit Key";
+	case 0x02:
+		return "Remote Unit Key";
+	case 0x03:
+		return "Debug Combination Key";
+	case 0x04:
+		return "Unauthenticated Combination Key";
+	case 0x05:
+		return "Authenticated Combination Key";
+	case 0x06:
+		return "Changed Combination Key";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *capability2str(uint8_t capability)
+{
+	switch (capability) {
+	case 0x00:
+		return "DisplayOnly";
+	case 0x01:
+		return "DisplayYesNo";
+	case 0x02:
+		return "KeyboardOnly";
+	case 0x03:
+		return "NoInputNoOutput";
+	default:
+		return "Reserved";
+	}
+}
+
+static char *authentication2str(uint8_t authentication)
+{
+	switch (authentication) {
+	case 0x00:
+		return "No Bonding (No MITM Protection)";
+	case 0x01:
+		return "No Bonding (MITM Protection)";
+	case 0x02:
+		return "Dedicated Bonding (No MITM Protection)";
+	case 0x03:
+		return "Dedicated Bonding (MITM Protection)";
+	case 0x04:
+		return "General Bonding (No MITM Protection)";
+	case 0x05:
+		return "General Bonding (MITM Protection)";
+	default:
+		return "Reserved";
+	}
+}
+
 static inline void ext_inquiry_response_dump(int level, struct frame *frm)
 {
 	void *ptr = frm->ptr;
@@ -479,14 +596,34 @@ static inline void ext_inquiry_response_dump(int level, struct frame *frm)
 		length--;
 
 		switch (type) {
+		case 0x01:
+			p_indent(level, frm);
+			printf("Flags:");
+			for (i = 0; i < length; i++)
+				printf(" 0x%2.2x", *((uint8_t *) (frm->ptr + i)));
+			printf("\n");
+			break;
+
+		case 0x02:
+		case 0x03:
+			p_indent(level, frm);
+			printf("%s service classes:",
+					type == 0x02 ? "Shortened" : "Complete");
+			for (i = 0; i < length / 2; i++) {
+				uint16_t val = btohs(bt_get_unaligned((uint16_t *) (frm->ptr + (i * 2))));
+				printf(" 0x%4.4x", val);
+			}
+			printf("\n");
+			break;
+
 		case 0x08:
 		case 0x09:
 			str = malloc(length + 1);
 			if (str) {
 				snprintf(str, length + 1, "%s", (char *) frm->ptr);
 				for (i = 0; i < length; i++)
-				if (!isprint(str[i]))
-					str[i] = '.';
+					if (!isprint(str[i]))
+						str[i] = '.';
 				p_indent(level, frm);
 				printf("%s local name: \'%s\'\n",
 					type == 0x08 ? "Shortened" : "Complete", str);
@@ -494,6 +631,11 @@ static inline void ext_inquiry_response_dump(int level, struct frame *frm)
 			}
 			break;
 
+		case 0x0a:
+			p_indent(level, frm);
+			printf("TX power level: %d\n", *((uint8_t *) frm->ptr));
+			break;
+
 		default:
 			p_indent(level, frm);
 			printf("Unknown type 0x%02x with %d bytes data\n",
@@ -511,16 +653,6 @@ static inline void ext_inquiry_response_dump(int level, struct frame *frm)
 	frm->len = len + (EXTENDED_INQUIRY_INFO_SIZE - INQUIRY_INFO_WITH_RSSI_SIZE);
 }
 
-static inline void generic_command_dump(int level, struct frame *frm)
-{
-	uint16_t handle = btohs(htons(get_u16(frm)));
-
-	p_indent(level, frm);
-	printf("handle %d\n", handle);
-
-	raw_dump(level, frm);
-}
-
 static inline void bdaddr_command_dump(int level, struct frame *frm)
 {
 	bdaddr_t *bdaddr = frm->ptr;
@@ -536,6 +668,24 @@ static inline void bdaddr_command_dump(int level, struct frame *frm)
 	raw_dump(level, frm);
 }
 
+static inline void generic_command_dump(int level, struct frame *frm)
+{
+	uint16_t handle = btohs(htons(get_u16(frm)));
+
+	p_indent(level, frm);
+	printf("handle %d\n", handle);
+
+	raw_dump(level, frm);
+}
+
+static inline void generic_write_mode_dump(int level, struct frame *frm)
+{
+	uint8_t mode = get_u8(frm);
+
+	p_indent(level, frm);
+	printf("mode 0x%2.2x\n", mode);
+}
+
 static inline void inquiry_dump(int level, struct frame *frm)
 {
 	inquiry_cp *cp = frm->ptr;
@@ -672,6 +822,59 @@ static inline void pin_code_neg_reply_dump(int level, struct frame *frm)
 	printf("bdaddr %s\n", addr);
 }
 
+static inline void user_passkey_reply_dump(int level, struct frame *frm)
+{
+	user_passkey_reply_cp *cp = frm->ptr;
+	char addr[18];
+
+	p_indent(level, frm);
+	p_ba2str(&cp->bdaddr, addr);
+	printf("bdaddr %s passkey %d\n", addr, btohl(cp->passkey));
+}
+
+static inline void remote_oob_data_reply_dump(int level, struct frame *frm)
+{
+	remote_oob_data_reply_cp *cp = frm->ptr;
+	char addr[18];
+	int i;
+
+	p_indent(level, frm);
+	p_ba2str(&cp->bdaddr, addr);
+	printf("bdaddr %s\n", addr);
+
+	p_indent(level, frm);
+	printf("hash 0x");
+	for (i = 0; i < 16; i++)
+		printf("%02x", cp->hash[i]);
+	printf("\n");
+
+	p_indent(level, frm);
+	printf("randomizer 0x");
+	for (i = 0; i < 16; i++)
+			printf("%02x", cp->randomizer[i]);
+	printf("\n");
+}
+
+static inline void io_capability_reply_dump(int level, struct frame *frm)
+{
+	io_capability_reply_cp *cp = frm->ptr;
+	char addr[18];
+
+	p_indent(level, frm);
+	p_ba2str(&cp->bdaddr, addr);
+	printf("bdaddr %s capability 0x%2.2x oob 0x%2.2x auth 0x%2.2x\n",
+					addr, cp->capability, cp->oob_data,
+							cp->authentication);
+
+	p_indent(level, frm);
+	printf("Capability: %s (OOB data %s)\n",
+			capability2str(cp->capability),
+			cp->oob_data == 0x00 ? "not present" : "available");
+
+	p_indent(level, frm);
+	printf("Authentication: %s\n", authentication2str(cp->authentication));
+}
+
 static inline void set_conn_encrypt_dump(int level, struct frame *frm)
 {
 	set_conn_encrypt_cp *cp = frm->ptr;
@@ -709,6 +912,15 @@ static inline void read_remote_ext_features_dump(int level, struct frame *frm)
 	printf("handle %d page %d\n", btohs(cp->handle), cp->page_num);
 }
 
+static inline void setup_sync_conn_dump(int level, struct frame *frm)
+{
+	setup_sync_conn_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d voice setting 0x%4.4x\n", btohs(cp->handle),
+						btohs(cp->voice_setting));
+}
+
 static inline void hold_mode_dump(int level, struct frame *frm)
 {
 	hold_mode_cp *cp = frm->ptr;
@@ -764,9 +976,25 @@ static inline void write_link_policy_dump(int level, struct frame *frm)
 	}
 }
 
-static inline void sniff_subrate_dump(int level, struct frame *frm)
+static inline void write_default_link_policy_dump(int level, struct frame *frm)
 {
-	sniff_subrate_cp *cp = frm->ptr;
+	uint16_t policy = btohs(htons(get_u16(frm)));
+	char *str;
+
+	p_indent(level, frm);
+	printf("policy 0x%2.2x\n", policy);
+
+	str = hci_lptostr(policy);
+	if (str) {
+		p_indent(level, frm);
+		printf("Link policy: %s\n", str);
+		free(str);
+	}
+}
+
+static inline void sniff_subrating_dump(int level, struct frame *frm)
+{
+	sniff_subrating_cp *cp = frm->ptr;
 
 	p_indent(level, frm);
 	printf("handle %d\n", btohs(cp->handle));
@@ -1017,6 +1245,40 @@ static inline void write_ext_inquiry_response_dump(int level, struct frame *frm)
 	ext_inquiry_response_dump(level, frm);
 }
 
+static inline void write_inquiry_transmit_power_level_dump(int level, struct frame *frm)
+{
+	write_inquiry_transmit_power_level_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("level %d\n", cp->level);
+}
+
+static inline void write_default_error_data_reporting_dump(int level, struct frame *frm)
+{
+	write_default_error_data_reporting_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("reporting %d\n", cp->reporting);
+}
+
+static inline void enhanced_flush_dump(int level, struct frame *frm)
+{
+	enhanced_flush_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d type %d\n", btohs(cp->handle), cp->type);
+}
+
+static inline void send_keypress_notify_dump(int level, struct frame *frm)
+{
+	send_keypress_notify_cp *cp = frm->ptr;
+	char addr[18];
+
+	p_indent(level, frm);
+	p_ba2str(&cp->bdaddr, addr);
+	printf("bdaddr %s type %d\n", addr, cp->type);
+}
+
 static inline void request_transmit_power_level_dump(int level, struct frame *frm)
 {
 	read_transmit_power_level_cp *cp = frm->ptr;
@@ -1091,7 +1353,7 @@ static inline void command_dump(int level, struct frame *frm)
 	frm->len -= HCI_COMMAND_HDR_SIZE;
 
 	if (ogf == OGF_VENDOR_CMD) {
-	       	if (ocf == 0 && get_manufacturer() == 10) {
+		if (ocf == 0 && get_manufacturer() == 10) {
 			csr_dump(level + 1, frm);
 			return;
 		}
@@ -1134,6 +1396,7 @@ static inline void command_dump(int level, struct frame *frm)
 			return;
 		case OCF_REJECT_CONN_REQ:
 		case OCF_REJECT_SYNC_CONN_REQ:
+		case OCF_IO_CAPABILITY_NEG_REPLY:
 			reject_conn_req_dump(level + 1, frm);
 			return;
 		case OCF_PIN_CODE_REPLY:
@@ -1144,8 +1407,21 @@ static inline void command_dump(int level, struct frame *frm)
 			return;
 		case OCF_PIN_CODE_NEG_REPLY:
 		case OCF_LINK_KEY_NEG_REPLY:
+		case OCF_USER_CONFIRM_REPLY:
+		case OCF_USER_CONFIRM_NEG_REPLY:
+		case OCF_USER_PASSKEY_NEG_REPLY:
+		case OCF_REMOTE_OOB_DATA_NEG_REPLY:
 			pin_code_neg_reply_dump(level + 1, frm);
 			return;
+		case OCF_USER_PASSKEY_REPLY:
+			user_passkey_reply_dump(level + 1, frm);
+			return;
+		case OCF_REMOTE_OOB_DATA_REPLY:
+			remote_oob_data_reply_dump(level + 1, frm);
+			return;
+		case OCF_IO_CAPABILITY_REPLY:
+			io_capability_reply_dump(level + 1, frm);
+			return;
 		case OCF_SET_CONN_ENCRYPT:
 			set_conn_encrypt_dump(level + 1, frm);
 			return;
@@ -1155,7 +1431,6 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_READ_REMOTE_VERSION:
 		case OCF_READ_CLOCK_OFFSET:
 		case OCF_READ_LMP_HANDLE:
-		case OCF_SETUP_SYNC_CONN:
 			generic_command_dump(level + 1, frm);
 			return;
 		case OCF_MASTER_LINK_KEY:
@@ -1167,6 +1442,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_REMOTE_NAME_REQ:
 			remote_name_req_dump(level + 1, frm);
 			return;
+		case OCF_SETUP_SYNC_CONN:
+			setup_sync_conn_dump(level + 1, frm);
+			return;
 		}
 		break;
 
@@ -1185,6 +1463,8 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_READ_LINK_POLICY:
 			generic_command_dump(level + 1, frm);
 			return;
+		case OCF_READ_DEFAULT_LINK_POLICY:
+			return;
 		case OCF_SWITCH_ROLE:
 			accept_conn_req_dump(level + 1, frm);
 			return;
@@ -1194,8 +1474,11 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_LINK_POLICY:
 			write_link_policy_dump(level + 1, frm);
 			return;
-		case OCF_SNIFF_SUBRATE:
-			sniff_subrate_dump(level + 1, frm);
+		case OCF_WRITE_DEFAULT_LINK_POLICY:
+			write_default_link_policy_dump(level + 1, frm);
+			return;
+		case OCF_SNIFF_SUBRATING:
+			sniff_subrating_dump(level + 1, frm);
 			return;
 		}
 		break;
@@ -1268,6 +1551,7 @@ static inline void command_dump(int level, struct frame *frm)
 			return;
 		case OCF_FLUSH:
 		case OCF_READ_LINK_SUPERVISION_TIMEOUT:
+		case OCF_REFRESH_ENCRYPTION_KEY:
 			generic_command_dump(level + 1, frm);
 			return;
 		case OCF_WRITE_LINK_SUPERVISION_TIMEOUT:
@@ -1276,6 +1560,21 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_EXT_INQUIRY_RESPONSE:
 			write_ext_inquiry_response_dump(level + 1, frm);
 			return;
+		case OCF_WRITE_SIMPLE_PAIRING_MODE:
+			generic_write_mode_dump(level + 1, frm);
+			return;
+		case OCF_WRITE_INQUIRY_TRANSMIT_POWER_LEVEL:
+			write_inquiry_transmit_power_level_dump(level + 1, frm);
+			return;
+		case OCF_WRITE_DEFAULT_ERROR_DATA_REPORTING:
+			write_default_error_data_reporting_dump(level + 1, frm);
+			return;
+		case OCF_ENHANCED_FLUSH:
+			enhanced_flush_dump(level + 1, frm);
+			return;
+		case OCF_SEND_KEYPRESS_NOTIFY:
+			send_keypress_notify_dump(level + 1, frm);
+			return;
 		}
 		break;
 
@@ -1299,6 +1598,15 @@ static inline void command_dump(int level, struct frame *frm)
 			return;
 		}
 		break;
+
+	case OGF_TESTING_CMD:
+		switch (ocf) {
+		case OCF_WRITE_LOOPBACK_MODE:
+		case OCF_WRITE_SIMPLE_PAIRING_DEBUG_MODE:
+			generic_write_mode_dump(level + 1, frm);
+			return;
+		}
+		break;
 	}
 
 	raw_dump(level, frm);
@@ -1366,6 +1674,65 @@ static inline void generic_response_dump(int level, struct frame *frm)
 	raw_dump(level, frm);
 }
 
+static inline void status_mode_dump(int level, struct frame *frm)
+{
+	uint8_t status = get_u8(frm);
+	uint8_t mode = get_u8(frm);
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x mode 0x%2.2x\n", status, mode);
+
+	if (status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(status));
+	}
+}
+
+static inline void read_link_policy_dump(int level, struct frame *frm)
+{
+	read_link_policy_rp *rp = frm->ptr;
+	uint16_t policy = btohs(rp->policy);
+	char *str;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x handle %d policy 0x%2.2x\n",
+				rp->status, btohs(rp->handle), policy);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	} else {
+		str = hci_lptostr(policy);
+		if (str) {
+			p_indent(level, frm);
+			printf("Link policy: %s\n", str);
+			free(str);
+		}
+	}
+}
+
+static inline void read_default_link_policy_dump(int level, struct frame *frm)
+{
+	uint8_t status = get_u8(frm);
+	uint16_t policy = btohs(htons(get_u16(frm)));
+	char *str;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x policy 0x%2.2x\n", status, policy);
+
+	if (status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(status));
+	} else {
+		str = hci_lptostr(policy);
+		if (str) {
+			p_indent(level, frm);
+			printf("Link policy: %s\n", str);
+			free(str);
+		}
+	}
+}
+
 static inline void read_pin_type_dump(int level, struct frame *frm)
 {
 	read_pin_type_rp *rp = frm->ptr;
@@ -1604,6 +1971,58 @@ static inline void read_ext_inquiry_response_dump(int level, struct frame *frm)
 	}
 }
 
+static inline void read_inquiry_transmit_power_level_dump(int level, struct frame *frm)
+{
+	read_inquiry_transmit_power_level_rp *rp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x level %d\n", rp->status, rp->level);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	}
+}
+
+static inline void read_default_error_data_reporting_dump(int level, struct frame *frm)
+{
+	read_default_error_data_reporting_rp *rp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x reporting %d\n", rp->status, rp->reporting);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	}
+}
+
+static inline void read_local_oob_data_dump(int level, struct frame *frm)
+{
+	read_local_oob_data_rp *rp = frm->ptr;
+	int i;
+
+	p_indent(level, frm);
+	printf("status 0x%2.2x\n", rp->status);
+
+	if (rp->status > 0) {
+		p_indent(level, frm);
+		printf("Error: %s\n", status2str(rp->status));
+	} else {
+		p_indent(level, frm);
+		printf("hash 0x");
+		for (i = 0; i < 16; i++)
+			printf("%02x", rp->hash[i]);
+		printf("\n");
+
+		p_indent(level, frm);
+		printf("randomizer 0x");
+		for (i = 0; i < 16; i++)
+			printf("%02x", rp->randomizer[i]);
+		printf("\n");
+	}
+}
+
 static inline void read_local_version_dump(int level, struct frame *frm)
 {
 	read_local_version_rp *rp = frm->ptr;
@@ -1820,6 +2239,14 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_LINK_KEY_REPLY:
 		case OCF_PIN_CODE_NEG_REPLY:
 		case OCF_LINK_KEY_NEG_REPLY:
+		case OCF_USER_CONFIRM_REPLY:
+		case OCF_USER_CONFIRM_NEG_REPLY:
+		case OCF_USER_PASSKEY_REPLY:
+		case OCF_USER_PASSKEY_NEG_REPLY:
+		case OCF_REMOTE_OOB_DATA_REPLY:
+		case OCF_REMOTE_OOB_DATA_NEG_REPLY:
+		case OCF_IO_CAPABILITY_REPLY:
+		case OCF_IO_CAPABILITY_NEG_REPLY:
 			bdaddr_response_dump(level, frm);
 			return;
 		}
@@ -1827,10 +2254,19 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 
 	case OGF_LINK_POLICY:
 		switch (ocf) {
+		case OCF_READ_LINK_POLICY:
+			read_link_policy_dump(level, frm);
+			return;
 		case OCF_WRITE_LINK_POLICY:
-		case OCF_SNIFF_SUBRATE:
+		case OCF_SNIFF_SUBRATING:
 			generic_response_dump(level, frm);
 			return;
+		case OCF_READ_DEFAULT_LINK_POLICY:
+			read_default_link_policy_dump(level, frm);
+			return;
+		case OCF_WRITE_DEFAULT_LINK_POLICY:
+			status_response_dump(level, frm);
+			return;
 		}
 		break;
 
@@ -1889,6 +2325,18 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_READ_EXT_INQUIRY_RESPONSE:
 			read_ext_inquiry_response_dump(level, frm);
 			return;
+		case OCF_READ_INQUIRY_TRANSMIT_POWER_LEVEL:
+			read_inquiry_transmit_power_level_dump(level, frm);
+			return;
+		case OCF_READ_DEFAULT_ERROR_DATA_REPORTING:
+			read_default_error_data_reporting_dump(level, frm);
+			return;
+		case OCF_READ_LOCAL_OOB_DATA:
+			read_local_oob_data_dump(level, frm);
+			return;
+		case OCF_READ_SIMPLE_PAIRING_MODE:
+			status_mode_dump(level, frm);
+			return;
 		case OCF_FLUSH:
 		case OCF_WRITE_LINK_SUPERVISION_TIMEOUT:
 			generic_response_dump(level, frm);
@@ -1914,8 +2362,13 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 		case OCF_WRITE_AFH_MODE:
 		case OCF_SET_AFH_CLASSIFICATION:
 		case OCF_WRITE_EXT_INQUIRY_RESPONSE:
+		case OCF_WRITE_SIMPLE_PAIRING_MODE:
+		case OCF_WRITE_INQUIRY_TRANSMIT_POWER_LEVEL:
+		case OCF_WRITE_DEFAULT_ERROR_DATA_REPORTING:
 		case OCF_SET_CONTROLLER_TO_HOST_FC:
 		case OCF_HOST_BUFFER_SIZE:
+		case OCF_REFRESH_ENCRYPTION_KEY:
+		case OCF_SEND_KEYPRESS_NOTIFY:
 			status_response_dump(level, frm);
 			return;
 		}
@@ -1964,6 +2417,19 @@ static inline void cmd_complete_dump(int level, struct frame *frm)
 			return;
 		}
 		break;
+
+	case OGF_TESTING_CMD:
+		switch (ocf) {
+		case OCF_READ_LOOPBACK_MODE:
+			status_mode_dump(level, frm);
+			return;
+		case OCF_WRITE_LOOPBACK_MODE:
+		case OCF_ENABLE_DEVICE_UNDER_TEST_MODE:
+		case OCF_WRITE_SIMPLE_PAIRING_DEBUG_MODE:
+			status_response_dump(level, frm);
+			return;
+		}
+		break;
 	}
 
 	raw_dump(level, frm);
@@ -2028,7 +2494,7 @@ static inline void conn_complete_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d bdaddr %s type %s encrypt 0x%2.2x\n",
 			evt->status, btohs(evt->handle), addr,
-			evt->link_type == 1 ? "ACL" : "SCO", evt->encr_mode);
+			linktype2str(evt->link_type), evt->encr_mode);
 
 	if (evt->status > 0) {
 		p_indent(level, frm);
@@ -2046,7 +2512,7 @@ static inline void conn_request_dump(int level, struct frame *frm)
 	p_indent(level, frm);
 	printf("bdaddr %s class 0x%2.2x%2.2x%2.2x type %s\n",
 			addr, evt->dev_class[2], evt->dev_class[1],
-			evt->dev_class[0], evt->link_type == 1 ? "ACL" : "SCO");
+			evt->dev_class[0], linktype2str(evt->link_type));
 }
 
 static inline void disconn_complete_dump(int level, struct frame *frm)
@@ -2246,6 +2712,9 @@ static inline void link_key_notify_dump(int level, struct frame *frm)
 		else
 			printf("%2.2X", evt->link_key[i]);
 	printf(" type %d\n", evt->key_type);
+
+	p_indent(level, frm);
+	printf("Type: %s\n", keytype2str(evt->key_type));
 }
 
 static inline void max_slots_change_dump(int level, struct frame *frm)
@@ -2261,7 +2730,7 @@ static inline void data_buffer_overflow_dump(int level, struct frame *frm)
 	evt_data_buffer_overflow *evt = frm->ptr;
 
 	p_indent(level, frm);
-	printf("type %s\n", evt->link_type == 1 ? "ACL" : "SCO");
+	printf("type %s\n", linktype2str(evt->link_type));
 }
 
 static inline void read_clock_offset_complete_dump(int level, struct frame *frm)
@@ -2433,9 +2902,9 @@ static inline void sync_conn_changed_dump(int level, struct frame *frm)
 	}
 }
 
-static inline void sniff_subrate_event_dump(int level, struct frame *frm)
+static inline void sniff_subrating_event_dump(int level, struct frame *frm)
 {
-	evt_sniff_subrate *evt = frm->ptr;
+	evt_sniff_subrating *evt = frm->ptr;
 
 	p_indent(level, frm);
 	printf("status 0x%2.2x handle %d\n", evt->status, btohs(evt->handle));
@@ -2479,15 +2948,52 @@ static inline void extended_inq_result_dump(int level, struct frame *frm)
 	}
 }
 
-static inline void link_supervision_timeout_change_dump(int level, struct frame *frm)
+static inline void link_supervision_timeout_changed_dump(int level, struct frame *frm)
 {
-	evt_link_supervision_timeout_change *evt = frm->ptr;
+	evt_link_supervision_timeout_changed *evt = frm->ptr;
 
 	p_indent(level, frm);
 	printf("handle %d timeout %d\n",
 				btohs(evt->handle), btohs(evt->timeout));
 }
 
+static inline void user_passkey_notify_dump(int level, struct frame *frm)
+{
+	evt_user_passkey_notify *evt = frm->ptr;
+	char addr[18];
+
+	p_indent(level, frm);
+	p_ba2str(&evt->bdaddr, addr);
+	printf("bdaddr %s passkey %d\n", addr, btohl(evt->passkey));
+}
+
+static inline void keypress_notify_dump(int level, struct frame *frm)
+{
+	evt_keypress_notify *evt = frm->ptr;
+	char addr[18];
+
+	p_indent(level, frm);
+	p_ba2str(&evt->bdaddr, addr);
+	printf("bdaddr %s type %d\n", addr, evt->type);
+}
+
+static inline void remote_host_features_notify_dump(int level, struct frame *frm)
+{
+	evt_remote_host_features_notify *evt = frm->ptr;
+	char addr[18];
+	int i;
+
+	p_indent(level, frm);
+	p_ba2str(&evt->bdaddr, addr);
+	printf("bdaddr %s\n", addr);
+
+	p_indent(level, frm);
+	printf("Features:");
+	for (i = 0; i < 8; i++)
+		printf(" 0x%2.2x", evt->features[i]);
+	printf("\n");
+}
+
 static inline void event_dump(int level, struct frame *frm)
 {
 	hci_event_hdr *hdr = frm->ptr;
@@ -2622,6 +3128,9 @@ static inline void event_dump(int level, struct frame *frm)
 		break;
 	case EVT_PIN_CODE_REQ:
 	case EVT_LINK_KEY_REQ:
+	case EVT_IO_CAPABILITY_REQUEST:
+	case EVT_USER_PASSKEY_REQUEST:
+	case EVT_REMOTE_OOB_DATA_REQUEST:
 		pin_code_req_dump(level + 1, frm);
 		break;
 	case EVT_LINK_KEY_NOTIFY:
@@ -2657,14 +3166,36 @@ static inline void event_dump(int level, struct frame *frm)
 	case EVT_SYNC_CONN_CHANGED:
 		sync_conn_changed_dump(level + 1, frm);
 		break;
-	case EVT_SNIFF_SUBRATE:
-		sniff_subrate_event_dump(level + 1, frm);
+	case EVT_SNIFF_SUBRATING:
+		sniff_subrating_event_dump(level + 1, frm);
 		break;
 	case EVT_EXTENDED_INQUIRY_RESULT:
 		extended_inq_result_dump(level + 1, frm);
 		break;
-	case EVT_LINK_SUPERVISION_TIMEOUT_CHANGE:
-		link_supervision_timeout_change_dump(level + 1, frm);
+	case EVT_ENCRYPTION_KEY_REFRESH_COMPLETE:
+		generic_response_dump(level + 1, frm);
+		break;
+	case EVT_SIMPLE_PAIRING_COMPLETE:
+		bdaddr_response_dump(level + 1, frm);
+		break;
+	case EVT_LINK_SUPERVISION_TIMEOUT_CHANGED:
+		link_supervision_timeout_changed_dump(level + 1, frm);
+		break;
+	case EVT_ENHANCED_FLUSH_COMPLETE:
+		generic_command_dump(level + 1, frm);
+		break;
+	case EVT_IO_CAPABILITY_RESPONSE:
+		io_capability_reply_dump(level + 1, frm);
+		break;
+	case EVT_USER_CONFIRM_REQUEST:
+	case EVT_USER_PASSKEY_NOTIFY:
+		user_passkey_notify_dump(level + 1, frm);
+		break;
+	case EVT_KEYPRESS_NOTIFY:
+		keypress_notify_dump(level + 1, frm);
+		break;
+	case EVT_REMOTE_HOST_FEATURES_NOTIFY:
+		remote_host_features_notify_dump(level + 1, frm);
 		break;
 	default:
 		raw_dump(level, frm);
@@ -2701,6 +3232,7 @@ static inline void sco_dump(int level, struct frame *frm)
 {
 	hci_sco_hdr *hdr = (void *) frm->ptr;
 	uint16_t handle = btohs(hdr->handle);
+	uint8_t flags = acl_flags(handle);
 	int len;
 
 	if (frm->audio_fd > fileno(stderr))
@@ -2708,8 +3240,8 @@ static inline void sco_dump(int level, struct frame *frm)
 
 	if (!p_filter(FILT_SCO)) {
 		p_indent(level, frm);
-		printf("SCO data: handle %d dlen %d\n",
-					acl_handle(handle), hdr->dlen);
+		printf("SCO data: handle %d flags 0x%2.2x dlen %d\n",
+				acl_handle(handle), flags, hdr->dlen);
 		level++;
 
 		frm->ptr += HCI_SCO_HDR_SIZE;
